import { useState, useEffect, useRef } from 'react'
import { aiSearch, paperOutline, outlineToPaper, download } from '@/api/wenku'
import { useAsyncTask } from './useAsyncTask'

/**
 * @typedef {Object} Message
 * @property {string} content
 * @property {'user' | 'ai'} role
 * @property {Array<{title: string, url: string}>} [searchReferList]
 * @property {string} [userQuery]
 * @property {Chapter[]} [chapters]
 * @property {string} [logID]
 * @property {string} [queryID]
 */

/**
 * @typedef {Object} Chapter
 * @property {string} title
 * @property {number} level
 * @property {string} desc
 * @property {string} chapter
 */

/**
 * @typedef {Object} Paper
 * @property {string} content
 * @property {string} [downloadLink]
 * @property {string} [docID]
 */

/**
 * @typedef {Object} DownloadResult
 * @property {string} download_link
 */

// AI搜索
export function useSearch() {
  const [messages, setMessages] = useState([])

  const { isLoading, execute } = useAsyncTask(async (userQuery) => {
    setMessages((prevMessages) => [...prevMessages, { content: userQuery, role: 'user' }])
    const currentMessage = { content: '', searchReferList: [], role: 'ai', userQuery }

    await aiSearch({
      body: JSON.stringify({ userQuery }),
      onmessage(message) {
        try {
          const data = JSON.parse(message.data)
          const content = data.raw?.content || ''
          const searchReferList = data.raw?.searchReferList || []
          currentMessage.content += content
          if (searchReferList.length) {
            currentMessage.searchReferList.push(...searchReferList)
            setMessages((prevMessages) => {
              const lastMessage = prevMessages[prevMessages.length - 1]
              if (lastMessage?.role === 'ai') {
                return [...prevMessages.slice(0, -1), currentMessage]
              }
              return [...prevMessages, currentMessage]
            })
          }
        } catch (error) {
          console.error('Error parsing message:', error)
        }
      },
    })
  })

  return { messages, setMessages, isLoading, query: execute }
}
function parseInput(inputStr) {
  const result = []
  const regex = /^(#+)\s*(.+?)\s*(?=\n|【描述】|$)/
  const descriptionRegex = /【描述】(.*)/

  const sections = []
  let level = 0

  const lines = inputStr.split('\n')
  lines.forEach((line) => {
    const headerMatch = line.match(regex)
    const descriptionMatch = line.match(descriptionRegex)

    if (headerMatch) {
      const header = headerMatch[1] // Capture the number of `#`
      const title = headerMatch[2].trim() // Capture the title
      const newLevel = header.length

      // Adjust section numbering based on levels
      if (newLevel > level) {
        sections.push(1) // Add new sub-level
      } else if (newLevel === level) {
        sections[sections.length - 1]++ // Increment current level
      } else {
        sections.splice(newLevel - 1) // Drop deeper levels
        sections[sections.length - 1]++
      }

      level = newLevel // Update current level

      // Create chapter number and entry
      const chapter = sections.join('.')
      result.push({
        title,
        level,
        desc: '', // Will be updated if a description follows
        chapter: level === 1 ? '标题' : `第${chapter}章`,
      })
    } else if (descriptionMatch) {
      const desc = descriptionMatch[1].trim()
      if (result.length > 0) {
        result[result.length - 1].desc = desc // Attach description to the last entry
      }
    }
  })

  return result
}

/**
 * 论文生成全流程管理
 * @returns {{
 *   messages: Message[],
 *   paper: Paper,
 *   isLoading: boolean,
 *   generateOutline: (userQuery: string) => Promise<void>,
 *   generatePaper: (data: any) => Promise<Paper>,
 *   downloadPaper: (docID: string) => Promise<string>
 * }}
 */
export function usePaper() {
  const [messages, setMessages] = useState([])
  const [paper, setPaper] = useState(null)
  const [downloadLink, setDownloadLink] = useState('')
  const timerRef = useRef(null)

  const clearTimer = () => {
    if (timerRef.current) {
      clearTimeout(timerRef.current)
      timerRef.current = null
    }
  }

  const { isLoading: isOutlineLoading, execute: generateOutline } = useAsyncTask(async (userQuery) => {
    setMessages((prevMessages) => [...prevMessages, { content: userQuery, role: 'user' }])
    const currentMessage = { content: '', role: 'ai', userQuery, chapters: [], logID: '', queryID: '' }

    await paperOutline({
      body: JSON.stringify({ userQuery }),
      onmessage(message) {
        try {
          const data = JSON.parse(message.data)
          const content = data.raw?.data || ''
          if (data.logID) currentMessage.logID = data.logID
          if (data.queryID) currentMessage.queryID = data.queryID
          currentMessage.content += content
          currentMessage.chapters = parseInput(currentMessage.content)
          if (content) {
            setMessages((prevMessages) => {
              const lastMessage = prevMessages[prevMessages.length - 1]
              if (lastMessage?.role === 'ai') {
                return [...prevMessages.slice(0, -1), currentMessage]
              }
              return [...prevMessages, currentMessage]
            })
          }
        } catch (error) {
          console.error('Error parsing message:', error)
        }
      },
    })
  })

  const { isLoading: isPaperLoading, execute: generatePaper } = useAsyncTask(async (data) => {
    const { raw } = await outlineToPaper(data)
    setPaper(raw)
    const downloadLink = await downloadWithPolling({
      docID: raw.docID,
    })
    setPaper({ ...raw, downloadLink })
    return { ...raw, downloadLink }
  })

  const { isLoading: isDownloadLoading, execute: downloadPaper } = useAsyncTask(async (docID) => {
    const { raw } = await download({ docID })
    if (raw) {
      setDownloadLink(raw.download_link)
      return raw.download_link
    }
  })

  const downloadWithPolling = async (data, interval = 3000) => {
    const poll = async (resolve, reject) => {
      try {
        const link = await downloadPaper(data.docID)
        if (link) {
          resolve(link)
          return
        }
      } catch (error) {
        reject(error)
        return
      }

      timerRef.current = setTimeout(() => poll(resolve, reject), interval)
    }

    return new Promise(poll)
  }

  useEffect(() => {
    return () => clearTimer()
  }, [])

  return {
    messages,
    setMessages,
    paper,
    downloadLink,
    isLoading: isOutlineLoading || isPaperLoading || isDownloadLoading,
    generateOutline,
    generatePaper,
    downloadPaper,
  }
}
